from __future__ import annotations

import typing as _t

import typing_extensions as _te


@_te.final
class bool_:
    def as_(self, cls: _t.Type[_CastableFromBool]) -> _CastableFromBool:
        from .integer import (BaseSignedInteger,
                              BaseUnsignedInteger)

        if issubclass(cls, type(self)):
            return self
        elif issubclass(cls, (BaseSignedInteger, BaseUnsignedInteger)):
            return cls(int(self._value))
        else:
            raise TypeError(cls)

    _value: bool

    __slots__ = '_value',

    def __new__(cls, _value: bool) -> bool_:
        if not isinstance(_value, bool):
            raise TypeError(type(_value))
        self = super().__new__(cls)
        self._value = _value
        return self

    def __bool__(self) -> bool:
        return self._value

    @_t.overload
    def __eq__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        return (bool_(self._value is other._value)
                if isinstance(other, bool_)
                else NotImplemented)

    @_t.overload
    def __ge__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        return (bool_(self._value >= other._value)
                if isinstance(other, bool_)
                else NotImplemented)

    @_t.overload
    def __gt__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        return (bool_(self._value > other._value)
                if isinstance(other, bool_)
                else NotImplemented)

    @_t.overload
    def __le__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        return (bool_(self._value <= other._value)
                if isinstance(other, bool_)
                else NotImplemented)

    @_t.overload
    def __lt__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        return (bool_(self._value < other._value)
                if isinstance(other, bool_)
                else NotImplemented)

    def __repr__(self) -> str:
        return f'{type(self).__qualname__}({self._value!r})'

    def __str__(self) -> str:
        return 'true' if self._value else 'false'


if _t.TYPE_CHECKING:
    from .integer import (BaseSignedInteger as _BaseSignedInteger,
                          BaseUnsignedInteger as _BaseUnsignedInteger)

    _CastableFromBool = _t.TypeVar('_CastableFromBool', bool_,
                                   _BaseSignedInteger,
                                   _BaseUnsignedInteger)
_T = _t.TypeVar('_T')


def try_construct_bool_(value: _T) -> _t.Union[_T, bool_]:
    return bool_(value) if isinstance(value, bool) else value



from __future__ import annotations

import typing as _t

import typing_extensions as _te

from ._core.bool_ import (bool_ as _bool,
                          try_construct_bool_ as _try_construct_bool)
from .option import (None_ as _None,
                     Some as _Some)

_E = _t.TypeVar('_E')
_E2 = _t.TypeVar('_E2')
_T = _t.TypeVar('_T')
_T2 = _t.TypeVar('_T2')


class Err(_t.Generic[_E]):
    def and_(self, _other: Result[_T, _E]) -> _te.Self:
        return self

    def and_then(self,
                 _function: _t.Callable[[_T], Result[_T2, _E]]) -> _te.Self:
        return self

    def err(self) -> _Some[_E]:
        return _Some(self._value)

    def expect(self, _message: str) -> _t.NoReturn:
        raise ValueError(f'{_message}: {self._value!r}')

    def expect_err(self, _message: str) -> _E:
        return self._value

    def is_err(self) -> _bool:
        return _bool(True)

    def is_ok(self) -> _bool:
        return _bool(False)

    def map(self, _function: _t.Callable[[_T], _T2]) -> _te.Self:
        return self

    def map_err(self, _function: _t.Callable[[_E], _E2]) -> Err[_E2]:
        return Err(_function(self._value))

    def map_or(self,
               _default: _T2,
               _function: _t.Callable[[_T], _T2]) -> _T2:
        return _default

    def map_or_else(self,
                    _default: _t.Callable[[_E], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        return _default(self._value)

    def ok(self) -> _None:
        return _None()

    def or_(self, _other: Result[_T, _E]) -> Result[_T, _E]:
        if not isinstance(_other, (Err, Ok)):
            raise TypeError(type(_other))
        return _other

    def or_else(
            self, _function: _t.Callable[[_E], Result[_T, _E2]]
    ) -> Result[_T, _E2]:
        result = _function(self._value)
        if not isinstance(result, (Err, Ok)):
            raise TypeError(type(result))
        return result

    def unwrap(self) -> _t.NoReturn:
        raise ValueError('Called `unwrap` on an `Err` value: '
                         f'{self._value!r}.')

    def unwrap_or(self, _default: _T) -> _T:
        return _default

    def unwrap_err(self) -> _E:
        return self._value

    def unwrap_or_else(self, _function: _t.Callable[[_E], _T]) -> _T:
        return _function(self._value)

    __module__ = 'rustpy.result'
    __slots__ = '_value',

    def __init__(self, value: _E) -> None:
        self._value = value

    def __init_subclass__(cls, **kwargs: _t.Any) -> _t.NoReturn:
        raise TypeError(f'type \'{cls.__module__}{cls.__qualname__}\' '
                        f'is not an acceptable base type')

    @_t.overload
    def __eq__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value == other._value)
                if isinstance(other, Err)
                else (_bool(not isinstance(other, Ok))
                      and NotImplemented))

    @_t.overload
    def __ge__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value >= other._value)
                if isinstance(other, Err)
                else _bool(isinstance(other, Ok)) or NotImplemented)

    @_t.overload
    def __gt__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value > other._value)
                if isinstance(other, Err)
                else _bool(isinstance(other, Ok)) or NotImplemented)

    @_t.overload
    def __le__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value <= other._value)
                if isinstance(other, Err)
                else _bool(not isinstance(other, Ok)) and NotImplemented)

    @_t.overload
    def __lt__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value < other._value)
                if isinstance(other, Err)
                else _bool(not isinstance(other, Ok)) and NotImplemented)

    def __repr__(self) -> str:
        return f'{type(self).__qualname__}({self._value!r})'


class Ok(_t.Generic[_T]):
    def and_(self, _other: Result[_T, _E]) -> Result[_T, _E]:
        if not isinstance(_other, (Err, Ok)):
            raise TypeError(type(_other))
        return _other

    def and_then(
            self, _function: _t.Callable[[_T], Result[_T2, _E]]
    ) -> Result[_T2, _E]:
        result = _function(self._value)
        if not isinstance(result, (Err, Ok)):
            raise TypeError(type(result))
        return result

    def err(self) -> _None:
        return _None()

    def expect(self, _message: str) -> _T:
        return self._value

    def expect_err(self, _message: str) -> _t.NoReturn:
        raise ValueError(f'{_message}: {self._value!r}')

    def is_err(self) -> _bool:
        return _bool(False)

    def is_ok(self) -> _bool:
        return _bool(True)

    def map(self, _function: _t.Callable[[_T], _T2]) -> Ok[_T2]:
        return Ok(_function(self._value))

    def map_err(self, _function: _t.Callable[[_E], _E2]) -> _te.Self:
        return self

    def map_or(self,
               _default: _T2,
               _function: _t.Callable[[_T], _T2]) -> _T2:
        return _function(self._value)

    def map_or_else(self,
                    _default: _t.Callable[[_E], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        return _function(self._value)

    def ok(self) -> _Some[_T]:
        return _Some(self._value)

    def or_(self, _other: Result[_T, _E]) -> _te.Self:
        return self

    def or_else(self,
                _function: _t.Callable[[_E], Result[_T, _E2]]) -> _te.Self:
        return self

    def unwrap(self) -> _T:
        return self._value

    def unwrap_err(self) -> _t.NoReturn:
        raise ValueError('Called `unwrap_err` on an `Ok` value: '
                         f'{self._value!r}.')

    def unwrap_or(self, _default: _T) -> _T:
        return self._value

    def unwrap_or_else(self, _function: _t.Callable[[_E], _T]) -> _T:
        return self._value

    __module__ = 'rustpy.result'
    __slots__ = '_value'

    def __init__(self, value: _T) -> None:
        self._value = value

    def __init_subclass__(cls, **kwargs: _t.Any) -> _t.NoReturn:
        raise TypeError(f'type \'{cls.__module__}{cls.__qualname__}\' '
                        f'is not an acceptable base type')

    @_t.overload
    def __eq__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value == other._value)
                if isinstance(other, Ok)
                else (_bool(not isinstance(other, Err))
                      and NotImplemented))

    @_t.overload
    def __ge__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value >= other._value)
                if isinstance(other, Ok)
                else _bool(not isinstance(other, Err)) and NotImplemented)

    @_t.overload
    def __gt__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value > other._value)
                if isinstance(other, Ok)
                else _bool(not isinstance(other, Err)) and NotImplemented)

    @_t.overload
    def __le__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value <= other._value)
                if isinstance(other, Ok)
                else _bool(isinstance(other, Err)) or NotImplemented)

    @_t.overload
    def __lt__(self, other: Result[_T, _E]) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value < other._value)
                if isinstance(other, Ok)
                else _bool(isinstance(other, Err)) or NotImplemented)

    def __repr__(self) -> str:
        return f'{type(self).__qualname__}({self._value!r})'


Result = _t.Union[Ok[_T], Err[_E]]



from __future__ import annotations

import typing as _t

import typing_extensions as _te

try:
    from ._crustpy import (None_,
                           Some)
except ImportError:
    from ._rustpy.option import (None_,
                                 Some)

if _t.TYPE_CHECKING:
    from .primitive import bool_ as _bool
    from .result import Result as _Result

_E = _t.TypeVar('_E')
_T = _t.TypeVar('_T')
_T2 = _t.TypeVar('_T2')

None_.__doc__ = ('Represents absense of value. '
                 'Implements :class:`Option` protocol.')

Some.__doc__ = 'Contains a value. Implements :class:`Option` protocol.'


class Option(_te.Protocol, _t.Generic[_T]):
    """Protocol of an optional value."""

    def and_(self, _other: Option[_T]) -> Option[_T]:
        """Returns self if it :meth:`is_none`, otherwise returns other."""

    def and_then(self,
                 _function: _t.Callable[[_T], Option[_T2]]) -> Option[_T2]:
        """
        Returns self if it :meth:`is_none`,
        otherwise returns the result of given function on the contained value.
        """

    def expect(self, _message: str) -> _T:
        """
        Returns the contained value
        or raises a :class:`ValueError` with given message if none.
        """

    def is_none(self) -> _bool:
        """Checks if the option does not contain a value."""

    def is_some(self) -> _bool:
        """Checks if the option contains a value."""

    def map(self, _function: _t.Callable[[_T], _T2]) -> Option[_T2]:
        """
        Returns self if it :meth:`is_none`,
        otherwise applies given function to a contained value.
        """

    def map_or(self, _default: _T2, _function: _t.Callable[[_T], _T2]) -> _T2:
        """
        Returns given default if self :meth:`is_none`,
        otherwise returns the result of given function on the contained value.
        """

    def map_or_else(self,
                    _default: _t.Callable[[], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        """
        Returns the result of given default function if self :meth:`is_none`,
        otherwise returns the result of given function on the contained value.
        """

    def ok_or(self, _err: _E) -> _Result[_T, _E]:
        """
        Returns the contained value wrapped in :class:`rustpy.result.Ok`
        if self :meth:`is_some`,
        otherwise returns given value wrapped in :class:`rustpy.result.Err`.
        """

    def ok_or_else(self, _function: _t.Callable[[], _E]) -> _Result[_T, _E]:
        """
        Returns the contained value wrapped in :class:`rustpy.result.Ok`
        if self :meth:`is_some`,
        otherwise returns the result of given function
        wrapped in :class:`rustpy.result.Err`.
        """

    def or_(self, _other: Option[_T]) -> Option[_T]:
        """
        Returns self if it :meth:`is_some`, otherwise returns other.
        """

    def or_else(self, _function: _t.Callable[[], Option[_T]]) -> Option[_T]:
        """
        Returns self if it :meth:`is_some`,
        otherwise returns the result of the given function.
        """

    def unwrap(self) -> _T:
        """Returns the contained value."""

    def unwrap_or(self, _default: _T) -> _T:
        """
        Returns the default if self :meth:`is_none`,
        otherwise returns the contained value.
        """

    def unwrap_or_else(self, _function: _t.Callable[[], _T]) -> _T:
        """
        Returns the result of default function if self :meth:`is_none`,
        otherwise returns the contained value.
        """

    def __bool__(self) -> _t.NoReturn:
        ...

    @_t.overload
    def __eq__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __ge__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __gt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __le__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        ...



import typing as _t

import typing_extensions as _te

from rustpy.primitive import bool_ as _bool
from rustpy.result import Result

_E = _t.TypeVar('_E')
_T = _t.TypeVar('_T')
_T2 = _t.TypeVar('_T2')


class Option(_te.Protocol, _t.Generic[_T]):
    def and_(self, _other: Option[_T]) -> Option[_T]:
        ...

    def and_then(self,
                 _function: _t.Callable[[_T], Option[_T2]]) -> Option[_T2]:
        ...

    def expect(self, _message: str) -> _T:
        ...

    def is_none(self) -> _bool:
        ...

    def is_some(self) -> _bool:
        ...

    def ok_or(self, _err: _E) -> Result[_T, _E]:
        ...

    def ok_or_else(self, _err: _t.Callable[[], _E]) -> Result[_T, _E]:
        ...

    def or_(self, _other: Option[_T]) -> Option[_T]:
        ...

    def or_else(self, _function: _t.Callable[[], Option[_T]]) -> Option[_T]:
        ...

    def map(self, _function: _t.Callable[[_T], _T2]) -> Some[_T2]:
        ...

    def map_or(self, _default: _T2, _function: _t.Callable[[_T], _T2]) -> _T2:
        ...

    def map_or_else(self,
                    _default: _t.Callable[[], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        ...

    def unwrap(self) -> _T:
        ...

    def unwrap_or(self, _default: _T) -> _T:
        ...

    def unwrap_or_else(self, _function: _t.Callable[[], _T]) -> _T:
        ...

    def __bool__(self) -> _t.NoReturn:
        ...

    @_t.overload
    def __eq__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __ge__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __gt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __le__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...


@_te.final
class None_(Option[_t.Any]):
    pass


@_te.final
class Some(Option[_T]):
    def __init__(self, _value: _T) -> None:
        ...



from __future__ import annotations

import typing as _t

import typing_extensions as _te

from ._core.bool_ import (bool_ as _bool,
                          try_construct_bool_ as _try_construct_bool)

if _t.TYPE_CHECKING:
    from .result import (Err,
                         Ok)

_E = _t.TypeVar('_E')
_T = _t.TypeVar('_T')
_T2 = _t.TypeVar('_T2')


class None_:
    def and_(self, _other: Option[_T]) -> _te.Self:
        return self

    def and_then(self, _function: _t.Callable[[_T], _T2]) -> _te.Self:
        return self

    def expect(self, _message: str) -> _t.NoReturn:
        raise ValueError(_message)

    def is_none(self) -> _bool:
        return _bool(True)

    def is_some(self) -> _bool:
        return _bool(False)

    def map(self, _function: _t.Callable[[_T], _T2]) -> None_:
        return self

    def map_or(self, _default: _T2, _function: _t.Callable[[_T], _T2]) -> _T2:
        return _default

    def map_or_else(self,
                    _default: _t.Callable[[], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        return _default()

    def ok_or(self, _err: _E) -> Err[_E]:
        from .result import Err
        return Err(_err)

    def ok_or_else(self, _err: _t.Callable[[], _E]) -> Err[_E]:
        from .result import Err
        return Err(_err())

    def or_(self, _other: Option[_T]) -> Option[_T]:
        if not isinstance(_other, (None_, Some)):
            raise TypeError(type(_other))
        return _other

    def or_else(self,
                _function: _t.Callable[[], Option[_T]]) -> Option[_T]:
        result = _function()
        if not isinstance(result, (None_, Some)):
            raise TypeError(type(result))
        return result

    def unwrap(self) -> _t.NoReturn:
        raise ValueError('Called `unwrap` on a `None` value.')

    def unwrap_or(self, _default: _T) -> _T:
        return _default

    def unwrap_or_else(self, _function: _t.Callable[[], _T]) -> _T:
        return _function()

    __module__ = 'rustpy.option'
    __slots__ = ()

    def __init_subclass__(cls, **kwargs: _t.Any) -> _t.NoReturn:
        raise TypeError(f'type \'{cls.__module__}{cls.__qualname__}\' '
                        f'is not an acceptable base type')

    @_t.overload
    def __eq__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        return (_bool(isinstance(other, None_))
                or (_bool(not isinstance(other, Some))
                    and NotImplemented))

    @_t.overload
    def __ge__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        return (_bool(isinstance(other, None_))
                or (_bool(not isinstance(other, Some))
                    and NotImplemented))

    @_t.overload
    def __gt__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        return _bool(not isinstance(other, (None_, Some))) and NotImplemented

    @_t.overload
    def __le__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        return _bool(isinstance(other, (None_, Some))) or NotImplemented

    @_t.overload
    def __lt__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        return (_bool(not isinstance(other, None_))
                and (_bool(isinstance(other, Some))
                     or NotImplemented))

    def __repr__(self) -> str:
        return f'{type(self).__qualname__}()'


class Some(_t.Generic[_T]):
    def and_(self, _other: Option[_T]) -> Option[_T]:
        if not isinstance(_other, (None_, Some)):
            raise TypeError(type(_other))
        return _other

    def and_then(self,
                 _function: _t.Callable[[_T], Option[_T2]]) -> Option[_T2]:
        result = _function(self._value)
        if not isinstance(result, (None_, Some)):
            raise TypeError(type(result))
        return result

    def expect(self, _message: str) -> _T:
        return self._value

    def is_none(self) -> _bool:
        return _bool(False)

    def is_some(self) -> _bool:
        return _bool(True)

    def ok_or(self, _err: _E) -> Ok[_T]:
        from .result import Ok
        return Ok(self._value)

    def ok_or_else(self, _err: _t.Callable[[], _E]) -> Ok[_T]:
        from .result import Ok
        return Ok(self._value)

    def or_(self, _other: Option[_T]) -> _te.Self:
        return self

    def or_else(self, _function: _t.Callable[[], Option[_T]]) -> _te.Self:
        return self

    def map(self, _function: _t.Callable[[_T], _T2]) -> Some[_T2]:
        return Some(_function(self._value))

    def map_or(self, _default: _T2, _function: _t.Callable[[_T], _T2]) -> _T2:
        return _function(self._value)

    def map_or_else(self,
                    _default: _t.Callable[[], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        return _function(self._value)

    def unwrap(self) -> _T:
        return self._value

    def unwrap_or(self, _default: _T) -> _T:
        return self._value

    def unwrap_or_else(self, _function: _t.Callable[[], _T]) -> _T:
        return self._value

    __module__ = 'rustpy.option'
    __slots__ = '_value',

    def __init__(self, value: _T) -> None:
        self._value = value

    def __init_subclass__(cls, **kwargs: _t.Any) -> _t.NoReturn:
        raise TypeError(f'type \'{cls.__module__}{cls.__qualname__}\' '
                        f'is not an acceptable base type')

    @_t.overload
    def __eq__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value == other._value)
                if isinstance(other, Some)
                else (_bool(not isinstance(other, None_))
                      and NotImplemented))

    @_t.overload
    def __ge__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value >= other._value)
                if isinstance(other, Some)
                else _bool(isinstance(other, None_)) or NotImplemented)

    @_t.overload
    def __gt__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value > other._value)
                if isinstance(other, Some)
                else _bool(isinstance(other, None_)) or NotImplemented)

    @_t.overload
    def __le__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value <= other._value)
                if isinstance(other, Some)
                else _bool(not isinstance(other, None_)) and NotImplemented)

    @_t.overload
    def __lt__(self, other: Option[_T]) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        return (_try_construct_bool(self._value < other._value)
                if isinstance(other, Some)
                else _bool(not isinstance(other, None_)) and NotImplemented)

    def __repr__(self) -> str:
        return f'{type(self).__qualname__}({self._value!r})'


Option = _t.Union[None_, Some[_T]]



from __future__ import annotations

import typing as _t

import typing_extensions as _te

from .bool_ import bool_ as _bool
from .wrapper import Wrapper as _Wrapper


class Ordered(_te.Protocol):
    @_t.overload
    def __ge__(self, other: _te.Self) -> bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __gt__(self, other: _te.Self) -> bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __le__(self, other: _te.Self) -> bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lt__(self, other: _te.Self) -> bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        ...


_OrderedT = _t.TypeVar('_OrderedT',
                       bound=Ordered)


class OrderedWrapper(_Wrapper[_OrderedT]):
    @_t.overload
    def __ge__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        return (_bool(self._value >= other._value)
                if isinstance(other, type(self))
                else NotImplemented)

    @_t.overload
    def __gt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        return (_bool(self._value > other._value)
                if isinstance(other, type(self))
                else NotImplemented)

    @_t.overload
    def __le__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        return (_bool(self._value <= other._value)
                if isinstance(other, type(self))
                else NotImplemented)

    @_t.overload
    def __lt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        return (_bool(self._value < other._value)
                if isinstance(other, type(self))
                else NotImplemented)



from __future__ import annotations

import math as _math
import typing as _t

import typing_extensions as _te

from .bool_ import bool_ as _bool
from .number import NumberWrapper as _NumberWrapper
from .utils import (floor_division_quotient as _floor_division_quotient,
                    floor_division_remainder as _floor_division_remainder,
                    trunc_division_quotient as _trunc_division_quotient,
                    trunc_division_remainder as _trunc_division_remainder)

if _t.TYPE_CHECKING:
    from rustpy._rustpy.primitive import (i32,
                                          u32)


class BaseFloat(_NumberWrapper[float]):
    DIGITS: _t.ClassVar[u32]
    EPSILON: _t.ClassVar[_te.Self]
    INFINITY: _t.ClassVar[_te.Self]
    MANTISSA_DIGITS: _t.ClassVar[u32]
    MAX: _t.ClassVar[_te.Self]
    MAX_10_EXP: _t.ClassVar[i32]
    MAX_EXP: _t.ClassVar[i32]
    MIN: _t.ClassVar[_te.Self]
    MIN_10_EXP: _t.ClassVar[i32]
    MIN_EXP: _t.ClassVar[i32]
    MIN_POSITIVE: _t.ClassVar[_te.Self]
    NAN: _t.ClassVar[_te.Self]
    NEG_INFINITY: _t.ClassVar[_te.Self]
    RADIX: _t.ClassVar[u32]

    def __new__(cls, _value: float) -> _te.Self:
        if not isinstance(_value, float):
            raise TypeError(type(_value))
        self = super().__new__(cls)
        self._value = _value
        return self

    def abs(self) -> _te.Self:
        return type(self)(abs(self._value))

    def ceil(self) -> _te.Self:
        fractional_part, whole_part = _math.modf(self._value)
        value = whole_part + _math.ceil(fractional_part)
        return type(self)(value)

    def div(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(type(divisor))
        return type(self)((_math.nan
                           if self._value == 0.0
                           else _math.copysign(_math.inf,
                                               self._value * divisor._value))
                          if divisor._value == 0.0
                          else _trunc_division_quotient(self._value,
                                                        divisor._value))

    def div_euclid(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(type(divisor))
        return type(self)((_math.nan
                           if self._value == 0.0
                           else _math.copysign(_math.inf,
                                               self._value * divisor._value))
                          if divisor._value == 0.0
                          else _floor_division_quotient(self._value,
                                                        divisor._value))

    def floor(self) -> _te.Self:
        fractional_part, whole_part = _math.modf(self._value)
        value = whole_part + _math.floor(fractional_part)
        return type(self)(value)

    def fract(self) -> _te.Self:
        value, _ = _math.modf(self._value)
        return type(self)(value)

    def is_finite(self) -> _bool:
        return _bool(_math.isfinite(self._value))

    def is_infinite(self) -> _bool:
        return _bool(_math.isinf(self._value))

    def is_nan(self) -> _bool:
        return _bool(_math.isnan(self._value))

    def neg(self) -> _te.Self:
        return type(self)(-self._value)

    def rem(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(type(self))
        return type(self)(_math.nan
                          if divisor._value == 0.0
                          else _trunc_division_remainder(self._value,
                                                         divisor._value))

    def rem_euclid(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(type(self))
        return type(self)(_math.nan
                          if divisor._value == 0.0
                          else _floor_division_remainder(self._value,
                                                         divisor._value))

    def round(self) -> _te.Self:
        fractional_part, whole_part = _math.modf(self._value)
        value = whole_part
        if abs(fractional_part) * 2.0 >= 1:
            value += 1.0 if fractional_part > 0.0 else -1.0
        return type(self)(value)

    def sub(self, other: _te.Self) -> _te.Self:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        return type(self)(self._value - other._value)

    def trunc(self) -> _te.Self:
        _, value = _math.modf(self._value)
        return type(self)(value)

    def __float__(self) -> float:
        return self._value

    @_t.overload
    def __mod__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __mod__(self, other: _t.Any) -> _t.Any:
        ...

    def __mod__(self, other: _t.Any) -> _t.Any:
        return (type(self)(_math.nan
                           if other._value == 0.0
                           else _trunc_division_remainder(self._value,
                                                          other._value))
                if isinstance(other, type(self))
                else NotImplemented)

    def __neg__(self) -> _te.Self:
        return type(self)(-self._value)

    def __repr__(self) -> str:
        return f'{type(self).__qualname__}({self._value})'

    def __str__(self) -> str:
        return f'{self._value}{type(self).__qualname__}'

    @_t.overload
    def __truediv__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __truediv__(self, other: _t.Any) -> _t.Any:
        ...

    def __truediv__(self, other: _t.Any) -> _t.Any:
        return (type(self)((_math.nan
                            if self._value == 0.0
                            else _math.copysign(_math.inf,
                                                self._value * other._value))
                           if other._value == 0.0
                           else _trunc_division_quotient(self._value,
                                                         other._value))
                if isinstance(other, type(self))
                else NotImplemented)



from __future__ import annotations

import operator as _operator
import typing as _t


def ceil_division_quotient(dividend: _t.Any, divisor: _t.Any) -> _t.Any:
    return -((-dividend) // divisor)


def ceil_division_remainder(dividend: _t.Any, divisor: _t.Any) -> _t.Any:
    return -((-dividend) % divisor)


floor_division_remainder = _operator.mod
floor_division_quotient = _operator.floordiv


def trunc_division_quotient(dividend: _t.Any, divisor: _t.Any) -> _t.Any:
    return (floor_division_quotient(dividend, divisor)
            if ((dividend < 0) is (divisor < 0))
            else ceil_division_quotient(dividend, divisor))


def trunc_division_remainder(dividend: _t.Any, divisor: _t.Any) -> _t.Any:
    return (floor_division_remainder(dividend, divisor)
            if ((dividend < 0) is (divisor < 0))
            else ceil_division_remainder(dividend, divisor))



from __future__ import annotations

import typing as _t

import typing_extensions as _te

try:
    from ._crustpy import (Err,
                           Ok)
except ImportError:
    from ._rustpy.result import (Err,
                                 Ok)

if _t.TYPE_CHECKING:
    from .option import Option as _Option
    from .primitive import bool_ as _bool

_E = _t.TypeVar('_E')
_E2 = _t.TypeVar('_E2')
_T = _t.TypeVar('_T')
_T2 = _t.TypeVar('_T2')

Err.__doc__ = ('Represents the error value. '
               'Implements :class:`Result` protocol.')
Ok.__doc__ = ('Represents the success value. '
              'Implements :class:`Result` protocol.')


class Result(_te.Protocol, _t.Generic[_T, _E]):
    def and_(self, _other: Result[_T2, _E]) -> Result[_T2, _E]:
        """Returns self if it :meth:`is_err`, otherwise returns other."""

    def and_then(
            self, _other: _t.Callable[[_T], Result[_T2, _E]]
    ) -> Result[_T2, _E]:
        """
        Returns self if it :meth:`is_err`,
        otherwise returns the result of given function on the success value.
        """

    def err(self) -> _Option[_E]:
        """
        Returns error value wrapped in :class:`rustpy.option.Some`
        if self :meth:`is_err`, otherwise returns :class:`rustpy.option.None_`.
        """

    def expect(self, _message: str) -> _T:
        """
        Returns the success value
        or raises a :class:`ValueError` with given message if error.
        """

    def expect_err(self, _message: str) -> _E:
        """
        Returns the error value
        or raises a :class:`ValueError` with given message if success.
        """

    def is_err(self) -> _bool:
        """Checks if the result is an error."""

    def is_ok(self) -> _bool:
        """Checks if the result is a success."""

    def map(self, _function: _t.Callable[[_T], _T2]) -> Result[_T2, _E]:
        """
        Returns self if it :meth:`is_err`,
        otherwise applies given function to a success value.
        """

    def map_err(self, _function: _t.Callable[[_E], _E2]) -> Result[_T, _E2]:
        """
        Returns self if it :meth:`is_ok`,
        otherwise applies given function to an error value.
        """

    def map_or(self, _default: _T2, _function: _t.Callable[[_T], _T2]) -> _T2:
        """
        Returns given default if self :meth:`is_err`,
        otherwise returns the result of given function on the success value.
        """

    def map_or_else(self,
                    _default: _t.Callable[[_E], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        """
        Returns the result of given default function on the error value
        if self :meth:`is_err`,
        otherwise returns the result of given function on the success value.
        """

    def ok(self) -> _Option[_T]:
        """
        Returns success value wrapped in :class:`rustpy.option.Some`
        if self :meth:`is_ok`, otherwise returns :class:`rustpy.option.None_`.
        """

    def or_(self, _other: Result[_T, _E2]) -> Result[_T, _E2]:
        """Returns self if it :meth:`is_ok`, otherwise returns other."""

    def or_else(self,
                _other: _t.Callable[[_E], Result[_T, _E2]]) -> Result[_T, _E2]:
        """
        Returns self if it :meth:`is_ok`,
        otherwise returns the result of given function on the success value.
        """

    def unwrap(self) -> _T:
        """Returns the success value."""

    def unwrap_err(self) -> _E:
        """Returns the error value."""

    def unwrap_or(self, _default: _T) -> _T:
        """
        Returns the default if self :meth:`is_err`,
        otherwise returns the success value.
        """

    def unwrap_or_else(self, _function: _t.Callable[[_E], _T]) -> _T:
        """
        Returns the result of default function on error value
        if self :meth:`is_err`, otherwise returns the success value.
        """

    def __bool__(self) -> _t.NoReturn:
        ...

    @_t.overload
    def __eq__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __ge__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __gt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __le__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    def __lt__(self, other: _t.Any) -> _t.Any:
        ...



from __future__ import annotations

import typing as _t

import typing_extensions as _te

from .bool_ import bool_ as _bool

_T = _t.TypeVar('_T')


class Wrapper(_t.Generic[_T]):
    _value: _T

    __module__ = 'rustpy.primitive'
    __slots__ = '_value',

    def __init__(self, _value: _T) -> None:
        self._value = _value

    def __bool__(self) -> _t.NoReturn:
        raise TypeError(f'Expected `{_bool.__qualname__}`, '
                        f'found `{type(self).__qualname__}`.')

    @_t.overload
    def __eq__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __eq__(self, other: _t.Any) -> _t.Any:
        return (_bool(self._value == other._value)
                if isinstance(other, type(self))
                else NotImplemented)



import typing as _t

import typing_extensions as _te

from rustpy.option import Option as _Option


@_te.final
class bool_:
    def as_(self, cls: _t.Type[_CastableFromBool]) -> _CastableFromBool:
        ...

    def __init__(self, _value: bool) -> None:
        ...


class _BaseFloat(_te.Protocol):
    DIGITS: _t.ClassVar[u32] = ...
    EPSILON: _t.ClassVar[_te.Self] = ...
    INFINITY: _t.ClassVar[_te.Self] = ...
    MANTISSA_DIGITS: _t.ClassVar[u32] = ...
    MAX: _t.ClassVar[_te.Self] = ...
    MAX_10_EXP: _t.ClassVar[i32] = ...
    MAX_EXP: _t.ClassVar[i32] = ...
    MIN: _t.ClassVar[_te.Self] = ...
    MIN_10_EXP: _t.ClassVar[i32] = ...
    MIN_EXP: _t.ClassVar[i32] = ...
    MIN_POSITIVE: _t.ClassVar[_te.Self] = ...
    NAN: _t.ClassVar[_te.Self] = ...
    NEG_INFINITY: _t.ClassVar[_te.Self] = ...
    RADIX: _t.ClassVar[u32] = ...

    @classmethod
    def from_be_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    @classmethod
    def from_le_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    @classmethod
    def from_ne_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    def abs(self) -> _te.Self:
        ...

    def add(self, other: _te.Self) -> _te.Self:
        ...

    def as_(self, cls: _t.Type[_PrimitiveNumberT]) -> _PrimitiveNumberT:
        ...

    def ceil(self) -> _te.Self:
        ...

    def div(self, divisor: _te.Self) -> _te.Self:
        ...

    def div_euclid(self, divisor: _te.Self) -> _te.Self:
        ...

    def floor(self) -> _te.Self:
        ...

    def fract(self) -> _te.Self:
        ...

    def is_finite(self) -> bool_:
        ...

    def is_infinite(self) -> bool_:
        ...

    def is_nan(self) -> bool_:
        ...

    def mul(self, other: _te.Self) -> _te.Self:
        ...

    def neg(self) -> _te.Self:
        ...

    def rem(self, divisor: _te.Self) -> _te.Self:
        ...

    def rem_euclid(self, divisor: _te.Self) -> _te.Self:
        ...

    def round(self) -> _te.Self:
        ...

    def sub(self, other: _te.Self) -> _te.Self:
        ...

    def to_be_bytes(self) -> bytes:
        ...

    def to_le_bytes(self) -> bytes:
        ...

    def to_ne_bytes(self) -> bytes:
        ...

    def trunc(self) -> _te.Self:
        ...

    @_t.overload
    def __add__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __add__(self, other: _t.Any) -> _t.Any:
        ...

    def __bool__(self) -> _t.NoReturn:
        ...

    @_t.overload
    def __eq__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    def __float__(self) -> float:
        ...

    @_t.overload
    def __ge__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __gt__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __init__(self, _value: float) -> None:
        ...

    @_t.overload
    def __le__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lt__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __mod__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __mod__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __mul__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __mul__(self, other: _t.Any) -> _t.Any:
        ...

    def __neg__(self) -> _te.Self:
        ...

    @_t.overload
    def __sub__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __sub__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __truediv__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __truediv__(self, other: _t.Any) -> _t.Any:
        ...


@_te.final
class f32(_BaseFloat):
    pass


@_te.final
class f64(_BaseFloat):
    pass


class _BaseInteger(_te.Protocol):
    BITS: _t.ClassVar[u32] = ...
    MAX: _t.ClassVar[_te.Self] = ...
    MIN: _t.ClassVar[_te.Self] = ...

    @classmethod
    def from_be_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    @classmethod
    def from_le_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    @classmethod
    def from_ne_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    def add(self, other: _te.Self) -> _te.Self:
        ...

    def as_(self, cls: _t.Type[_PrimitiveNumberT]) -> _PrimitiveNumberT:
        ...

    def checked_add(self, other: _te.Self) -> _Option[_te.Self]:
        ...

    def checked_div(self, other: _te.Self) -> _Option[_te.Self]:
        ...

    def checked_div_euclid(self, other: _te.Self) -> _Option[_te.Self]:
        ...

    def checked_mul(self, other: _te.Self) -> _Option[_te.Self]:
        ...

    def checked_rem(self, other: _te.Self) -> _Option[_te.Self]:
        ...

    def checked_rem_euclid(self, other: _te.Self) -> _Option[_te.Self]:
        ...

    def checked_sub(self, other: _te.Self) -> _Option[_te.Self]:
        ...

    def div(self, divisor: _te.Self) -> _te.Self:
        ...

    def div_euclid(self, divisor: _te.Self) -> _te.Self:
        ...

    def mul(self, other: _te.Self) -> _te.Self:
        ...

    def rem(self, divisor: _te.Self) -> _te.Self:
        ...

    def rem_euclid(self, divisor: _te.Self) -> _te.Self:
        ...

    def sub(self, other: _te.Self) -> _te.Self:
        ...

    def to_be_bytes(self) -> bytes:
        ...

    def to_le_bytes(self) -> bytes:
        ...

    def to_ne_bytes(self) -> bytes:
        ...

    @_t.overload
    def __add__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __add__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __and__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __and__(self, other: _t.Any) -> _t.Any:
        ...

    def __bool__(self) -> _t.NoReturn:
        ...

    @_t.overload
    def __eq__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __ge__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __gt__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    def __init__(self, _value: int) -> None:
        ...

    def __invert__(self) -> _te.Self:
        ...

    def __int__(self) -> int:
        ...

    @_t.overload
    def __le__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lt__(self, other: _te.Self) -> bool_:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lshift__(self, other: u32) -> _te.Self:
        ...

    @_t.overload
    def __lshift__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __mod__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __mod__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __mul__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __mul__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __rshift__(self, other: u32) -> _te.Self:
        ...

    @_t.overload
    def __rshift__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __sub__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __sub__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __truediv__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __truediv__(self, other: _t.Any) -> _t.Any:
        ...


class _BaseSignedInteger(_BaseInteger):
    def abs(self) -> _te.Self:
        ...

    def checked_abs(self) -> _Option[_te.Self]:
        ...

    def checked_neg(self) -> _Option[_te.Self]:
        ...

    def is_negative(self) -> bool_:
        ...

    def is_positive(self) -> bool_:
        ...

    def neg(self) -> _te.Self:
        ...

    def __neg__(self) -> _te.Self:
        ...


class _BaseUnsignedInteger(_BaseInteger):
    ...


@_te.final
class i128(_BaseSignedInteger):
    pass


@_te.final
class i16(_BaseSignedInteger):
    pass


@_te.final
class i32(_BaseSignedInteger):
    pass


@_te.final
class i64(_BaseSignedInteger):
    pass


@_te.final
class i8(_BaseSignedInteger):
    pass


@_te.final
class isize(_BaseSignedInteger):
    pass


@_te.final
class str_:
    pass


@_te.final
class u128(_BaseUnsignedInteger):
    pass


@_te.final
class u16(_BaseUnsignedInteger):
    pass


@_te.final
class u32(_BaseUnsignedInteger):
    pass


@_te.final
class u64(_BaseUnsignedInteger):
    pass


@_te.final
class u8(_BaseUnsignedInteger):
    pass


@_te.final
class usize(_BaseUnsignedInteger):
    pass


_CastableFromBool = _t.TypeVar('_CastableFromBool', _BaseInteger, bool_)
_PrimitiveNumberT = _t.TypeVar('_PrimitiveNumberT', _BaseInteger, _BaseFloat)



from __future__ import annotations

import math as _math
import struct as _struct
import typing as _t
from ctypes import c_float

import typing_extensions as _te

from ._core.bool_ import bool_ as _bool
from ._core.float import BaseFloat as _BaseFloat
from ._core.integer import (
    SIZE_BITS as _SIZE_BITS,
    BaseSignedInteger as _BaseSignedInteger,
    BaseUnsignedInteger as _BaseUnsignedInteger,
    signed_cls_to_max_value as _signed_cls_to_max_value,
    signed_cls_to_min_value as _signed_cls_to_min_value,
    u32_to_int as _u32_to_int,
    unsigned_cls_to_max_value as _unsigned_cls_to_max_value,
    unsigned_cls_to_min_value as _unsigned_cls_to_min_value
)

bool_ = _bool


class _CastableBaseSignedInteger(_BaseSignedInteger):
    def as_(self, cls: _t.Type[_PrimitiveNumberT]) -> _PrimitiveNumberT:
        if issubclass(cls, (_CastableBaseSignedInteger,
                            _CastableBaseUnsignedInteger)):
            return cls.from_le_bytes(
                    self.to_le_bytes()[:_u32_to_int(cls.BITS) // 8]
                    if self.BITS > cls.BITS
                    else self._value.to_bytes(_u32_to_int(cls.BITS) // 8,
                                              'little',
                                              signed=True)
            )
        elif issubclass(cls, _BaseFloat):
            return cls(float(self._value))
        else:
            raise TypeError(cls)


class _CastableBaseUnsignedInteger(_BaseUnsignedInteger):
    def as_(self, cls: _t.Type[_PrimitiveNumberT]) -> _PrimitiveNumberT:
        if issubclass(cls, (_CastableBaseSignedInteger,
                            _CastableBaseUnsignedInteger)):
            return cls.from_le_bytes(
                    self.to_le_bytes()[:_u32_to_int(cls.BITS) // 8]
                    if self.BITS > cls.BITS
                    else self._value.to_bytes(_u32_to_int(cls.BITS) // 8,
                                              'little')
            )
        elif issubclass(cls, _BaseFloat):
            return cls(float(self._value))
        else:
            raise TypeError(cls)


@_te.final
class u32(_CastableBaseUnsignedInteger):
    pass


u32.BITS = u32(32)
u32.MAX = _unsigned_cls_to_max_value(u32)
u32.MIN = u32(0)


@_te.final
class i8(_CastableBaseSignedInteger):
    pass


i8.BITS = u32(8)
i8.MAX = _signed_cls_to_max_value(i8)
i8.MIN = _signed_cls_to_min_value(i8)


@_te.final
class i16(_CastableBaseSignedInteger):
    pass


i16.BITS = u32(16)
i16.MAX = _signed_cls_to_max_value(i16)
i16.MIN = _signed_cls_to_min_value(i16)


@_te.final
class i32(_CastableBaseSignedInteger):
    pass


i32.BITS = u32(32)
i32.MAX = _signed_cls_to_max_value(i32)
i32.MIN = _signed_cls_to_min_value(i32)


@_te.final
class i64(_CastableBaseSignedInteger):
    pass


i64.BITS = u32(64)
i64.MAX = _signed_cls_to_max_value(i64)
i64.MIN = _signed_cls_to_min_value(i64)


@_te.final
class i128(_CastableBaseSignedInteger):
    pass


i128.BITS = u32(128)
i128.MAX = _signed_cls_to_max_value(i128)
i128.MIN = _signed_cls_to_min_value(i128)


@_te.final
class isize(_CastableBaseSignedInteger):
    pass


isize.BITS = u32(_SIZE_BITS)
isize.MAX = _signed_cls_to_max_value(isize)
isize.MIN = _signed_cls_to_min_value(isize)


@_te.final
class u8(_CastableBaseUnsignedInteger):
    pass


u8.BITS = u32(8)
u8.MAX = _unsigned_cls_to_max_value(u8)
u8.MIN = _unsigned_cls_to_min_value(u8)


@_te.final
class u16(_CastableBaseUnsignedInteger):
    pass


u16.BITS = u32(16)
u16.MAX = _unsigned_cls_to_max_value(u16)
u16.MIN = _unsigned_cls_to_min_value(u16)


@_te.final
class u64(_CastableBaseUnsignedInteger):
    pass


u64.BITS = u32(64)
u64.MAX = _unsigned_cls_to_max_value(u64)
u64.MIN = _unsigned_cls_to_min_value(u64)


@_te.final
class u128(_CastableBaseUnsignedInteger):
    pass


u128.BITS = u32(128)
u128.MAX = _unsigned_cls_to_max_value(u128)
u128.MIN = _unsigned_cls_to_min_value(u128)


@_te.final
class usize(_CastableBaseUnsignedInteger):
    pass


usize.BITS = u32(_SIZE_BITS)
usize.MAX = _unsigned_cls_to_max_value(usize)
usize.MIN = _unsigned_cls_to_min_value(usize)


@_te.final
class f32(_BaseFloat):
    @classmethod
    def from_be_bytes(cls, _bytes: bytes) -> _te.Self:
        try:
            value, = _struct.unpack('>f', _bytes)
        except _struct.error:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        else:
            return cls(value)

    @classmethod
    def from_le_bytes(cls, _bytes: bytes) -> _te.Self:
        try:
            value, = _struct.unpack('<f', _bytes)
        except _struct.error:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        else:
            return cls(value)

    @classmethod
    def from_ne_bytes(cls, _bytes: bytes) -> _te.Self:
        try:
            value, = _struct.unpack('=f', _bytes)
        except _struct.error:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        else:
            return cls(value)

    def as_(self, cls: _t.Type[_PrimitiveNumberT]) -> _PrimitiveNumberT:
        return _cast_float_as(self._value, cls)

    def to_be_bytes(self) -> bytes:
        return _struct.pack('>f', self._value)

    def to_le_bytes(self) -> bytes:
        return _struct.pack('<f', self._value)

    def to_ne_bytes(self) -> bytes:
        return _struct.pack('=f', self._value)

    def __init__(self, _value: float) -> None:
        self._value = c_float(_value).value


f32.DIGITS = u32(6)
f32.EPSILON = f32(1.1920929e-7)
f32.INFINITY = f32(_math.inf)
f32.MANTISSA_DIGITS = u32(24)
f32.MAX = f32(3.40282347e+38)
f32.MAX_10_EXP = i32(38)
f32.MAX_EXP = i32(128)
f32.MIN = f32(-3.40282347e+38)
f32.MIN_10_EXP = i32(-37)
f32.MIN_EXP = i32(-125)
f32.MIN_POSITIVE = f32(1.17549435e-38)
f32.NAN = f32(_math.nan)
f32.NEG_INFINITY = f32(-_math.inf)
f32.RADIX = u32(2)


@_te.final
class f64(_BaseFloat):
    @classmethod
    def from_be_bytes(cls, _bytes: bytes) -> _te.Self:
        try:
            value, = _struct.unpack('>d', _bytes)
            return cls(value)
        except _struct.error:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')

    @classmethod
    def from_le_bytes(cls, _bytes: bytes) -> _te.Self:
        try:
            value, = _struct.unpack('<d', _bytes)
            return cls(value)
        except _struct.error:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')

    @classmethod
    def from_ne_bytes(cls, _bytes: bytes) -> _te.Self:
        try:
            value, = _struct.unpack('=d', _bytes)
            return cls(value)
        except _struct.error:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')

    def as_(self, cls: _t.Type[_PrimitiveNumberT]) -> _PrimitiveNumberT:
        return _cast_float_as(self._value, cls)

    def to_be_bytes(self) -> bytes:
        return _struct.pack('>d', self._value)

    def to_le_bytes(self) -> bytes:
        return _struct.pack('<d', self._value)

    def to_ne_bytes(self) -> bytes:
        return _struct.pack('=d', self._value)


f64.DIGITS = u32(15)
f64.EPSILON = f64(2.2204460492503131e-16)
f64.INFINITY = f64(_math.inf)
f64.MANTISSA_DIGITS = u32(53)
f64.MAX = f64(1.7976931348623157e+308)
f64.MAX_10_EXP = i32(308)
f64.MAX_EXP = i32(1_024)
f64.MIN = f64(-1.7976931348623157e+308)
f64.MIN_10_EXP = i32(-307)
f64.MIN_EXP = i32(-1_021)
f64.MIN_POSITIVE = f64(2.2250738585072014e-308)
f64.NAN = f64(_math.nan)
f64.NEG_INFINITY = f64(-_math.inf)
f64.RADIX = u32(2)

_PrimitiveNumberT = _t.TypeVar(
        '_PrimitiveNumberT', _BaseFloat, _CastableBaseSignedInteger,
        _CastableBaseUnsignedInteger
)


def _cast_float_as(value: float,
                   cls: _t.Type[_PrimitiveNumberT]) -> _PrimitiveNumberT:
    if issubclass(cls, _BaseFloat):
        return cls(value)
    elif issubclass(cls, (_CastableBaseSignedInteger,
                          _CastableBaseUnsignedInteger)):
        try:
            return cls(int(value))
        except OverflowError:
            return cls.MIN if value < 0 else cls.MAX
        except ValueError:
            assert _math.isnan(value), value
            return cls(0)
    else:
        raise TypeError(cls)



try:
    from ._crustpy import (bool_,
                           f32,
                           f64,
                           i128,
                           i16,
                           i32,
                           i64,
                           i8,
                           isize,
                           u128,
                           u16,
                           u32,
                           u64,
                           u8,
                           usize)
except ImportError:
    from ._rustpy.primitive import (bool_,
                                    f32,
                                    f64,
                                    i128,
                                    i16,
                                    i32,
                                    i64,
                                    i8,
                                    isize,
                                    u128,
                                    u16,
                                    u32,
                                    u64,
                                    u8,
                                    usize)

bool_.__doc__ = (
    'Represents a value, which could only be either true or false. '
    'If you cast a bool into an integer, true will be 1 and false will be 0.'
)
f32.__doc__ = ('A 32-bit floating point type '
               '(specifically, the "binary32" type defined in IEEE 754-2008).')
f64.__doc__ = ('A 64-bit floating point type '
               '(specifically, the "binary64" type defined in IEEE 754-2008).')
i128.__doc__ = 'The 128-bit signed integer type.'
i64.__doc__ = 'The 64-bit signed integer type.'
i32.__doc__ = 'The 32-bit signed integer type.'
i16.__doc__ = 'The 16-bit signed integer type.'
i8.__doc__ = 'The 8-bit signed integer type.'
isize.__doc__ = 'The pointer-sized signed integer type.'
u128.__doc__ = 'The 128-bit unsigned integer type.'
u64.__doc__ = 'The 64-bit unsigned integer type.'
u32.__doc__ = 'The 32-bit unsigned integer type.'
u16.__doc__ = 'The 16-bit unsigned integer type.'
u8.__doc__ = 'The 8-bit unsigned integer type.'
usize.__doc__ = 'The pointer-sized unsigned integer type.'



import typing as _t

import typing_extensions as _te

from rustpy.option import Option as _Option
from rustpy.primitive import bool_ as _bool

_E = _t.TypeVar('_E')
_E2 = _t.TypeVar('_E2')
_T = _t.TypeVar('_T')
_T2 = _t.TypeVar('_T2')


class Result(_te.Protocol, _t.Generic[_T, _E]):
    def and_(self, _other: Result[_T2, _E]) -> Result[_T2, _E]:
        ...

    def and_then(
            self, _other: _t.Callable[[_T], Result[_T2, _E]]
    ) -> Result[_T2, _E]:
        ...

    def err(self) -> _Option[_E]:
        ...

    def expect(self, _message: str) -> _T:
        ...

    def expect_err(self, _message: str) -> _E:
        ...

    def is_err(self) -> _bool:
        ...

    def is_ok(self) -> _bool:
        ...

    def map(self, _function: _t.Callable[[_T], _T2]) -> Result[_T2, _E]:
        ...

    def map_err(self, _function: _t.Callable[[_E], _E2]) -> Result[_T, _E2]:
        ...

    def map_or(self, _default: _T2, _function: _t.Callable[[_T], _T2]) -> _T2:
        ...

    def map_or_else(self,
                    _default: _t.Callable[[_E], _T2],
                    _function: _t.Callable[[_T], _T2]) -> _T2:
        ...

    def ok(self) -> _Option[_T]:
        ...

    def or_(self, _other: Result[_T, _E2]) -> Result[_T, _E2]:
        ...

    def or_else(self,
                _other: _t.Callable[[_E], Result[_T, _E2]]) -> Result[_T, _E2]:
        ...

    def unwrap(self) -> _T:
        ...

    def unwrap_err(self) -> _E:
        ...

    def unwrap_or(self, _default: _T) -> _T:
        ...

    def unwrap_or_else(self, _function: _t.Callable[[_E], _T]) -> _T:
        ...

    def __bool__(self) -> _t.NoReturn:
        ...

    @_t.overload
    def __eq__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __eq__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __ge__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __ge__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __gt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __gt__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __le__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __le__(self, other: _t.Any) -> _t.Any:
        ...

    @_t.overload
    def __lt__(self, other: _te.Self) -> _bool:
        ...

    @_t.overload
    def __lt__(self, other: _t.Any) -> _t.Any:
        ...


@_te.final
class Err(Result[_t.Any, _E]):
    def __init__(self, _value: _E) -> None:
        ...


@_te.final
class Ok(Result[_T, _t.Any]):
    def __init__(self, _value: _T) -> None:
        ...



import platform
from pathlib import Path

from setuptools import (find_packages,
                        setup)

project_base_url = 'https://github.com/lycantropos/rustpy/'


def read_file(path_string: str) -> str:
    return Path(path_string).read_text(encoding='utf-8')


parameters = dict(packages=find_packages(exclude=('tests', 'tests.*')),
                  url=project_base_url,
                  download_url=project_base_url + 'archive/master.zip')
if platform.python_implementation() == 'CPython':
    from setuptools_rust import RustExtension

    parameters.update(rust_extensions=[RustExtension(f'rustpy._crustpy')],
                      include_package_data=True,
                      zip_safe=False)
setup(**parameters)



import typing as _t

import typing_extensions as _te

from .ordered import (Ordered as _Ordered,
                      OrderedWrapper as _OrderedWrapper)


class _Number(_Ordered, _te.Protocol):
    def __add__(self, other: _te.Self) -> _te.Self:
        ...

    def __mul__(self, other: _te.Self) -> _te.Self:
        ...

    def __sub__(self, other: _te.Self) -> _te.Self:
        ...


_NumberT = _t.TypeVar('_NumberT',
                      bound=_Number)


class NumberWrapper(_OrderedWrapper[_NumberT]):
    def add(self, other: _te.Self) -> _te.Self:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        return type(self)(self._value + other._value)

    def mul(self, other: _te.Self) -> _te.Self:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        return type(self)(self._value * other._value)

    def sub(self, other: _te.Self) -> _te.Self:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        return type(self)(self._value - other._value)

    @_t.overload
    def __add__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __add__(self, other: _t.Any) -> _t.Any:
        ...

    def __add__(self, other: _t.Any) -> _t.Any:
        return (type(self)(self._value + other._value)
                if isinstance(other, type(self))
                else NotImplemented)

    @_t.overload
    def __mul__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __mul__(self, other: _t.Any) -> _t.Any:
        ...

    def __mul__(self, other: _t.Any) -> _t.Any:
        return (type(self)(self._value * other._value)
                if isinstance(other, type(self))
                else NotImplemented)

    @_t.overload
    def __sub__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __sub__(self, other: _t.Any) -> _t.Any:
        ...

    def __sub__(self, other: _t.Any) -> _t.Any:
        return (type(self)(self._value - other._value)
                if isinstance(other, type(self))
                else NotImplemented)



from __future__ import annotations

import abc as _abc
import sys as _sys
import typing as _t

import typing_extensions as _te

from rustpy.option import (None_ as _None,
                           Option as _Option,
                           Some as _Some)
from .bool_ import bool_ as _bool
from .number import NumberWrapper as _NumberWrapper
from .utils import (floor_division_quotient as _floor_division_quotient,
                    floor_division_remainder as _floor_division_remainder,
                    trunc_division_quotient as _trunc_division_quotient,
                    trunc_division_remainder as _trunc_division_remainder)

if _t.TYPE_CHECKING:
    from rustpy._rustpy.primitive import u32

SIZE_BITS = (_sys.maxsize + 1).bit_length()
assert ((1 << (SIZE_BITS - 1)) - 1) == _sys.maxsize


class _BaseInteger(_abc.ABC, _NumberWrapper[int]):
    BITS: _t.ClassVar[u32]
    MAX: _t.ClassVar[_te.Self]
    MIN: _t.ClassVar[_te.Self]

    @classmethod
    @_abc.abstractmethod
    def from_be_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    @classmethod
    @_abc.abstractmethod
    def from_le_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    @classmethod
    @_abc.abstractmethod
    def from_ne_bytes(cls, _bytes: bytes) -> _te.Self:
        ...

    def checked_add(self, other: _te.Self) -> _Option[_te.Self]:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        try:
            return _Some(type(self)(self._value + other._value))
        except OverflowError:
            return _None()

    def checked_div(self, other: _te.Self) -> _Option[_te.Self]:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        try:
            return _Some(type(self)(_trunc_division_quotient(self._value,
                                                             other._value)))
        except (OverflowError, ZeroDivisionError):
            return _None()

    def checked_div_euclid(self, other: _te.Self) -> _Option[_te.Self]:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        try:
            return _Some(type(self)(_floor_division_quotient(self._value,
                                                             other._value)))
        except (OverflowError, ZeroDivisionError):
            return _None()

    def checked_mul(self, other: _te.Self) -> _Option[_te.Self]:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        try:
            return _Some(type(self)(self._value * other._value))
        except OverflowError:
            return _None()

    def checked_rem(self, other: _te.Self) -> _Option[_te.Self]:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        try:
            return _Some(type(self)(_trunc_division_remainder(self._value,
                                                              other._value)))
        except (OverflowError, ZeroDivisionError):
            return _None()

    def checked_rem_euclid(self, other: _te.Self) -> _Option[_te.Self]:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        try:
            return _Some(type(self)(_floor_division_remainder(self._value,
                                                              other._value)))
        except (OverflowError, ZeroDivisionError):
            return _None()

    def checked_sub(self, other: _te.Self) -> _Option[_te.Self]:
        if not isinstance(other, type(self)):
            raise TypeError(type(other))
        try:
            return _Some(type(self)(self._value - other._value))
        except OverflowError:
            return _None()

    def div(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(type(divisor))
        return type(self)(_trunc_division_quotient(self._value,
                                                   divisor._value))

    def div_euclid(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(type(divisor))
        return type(self)(_floor_division_quotient(self._value,
                                                   divisor._value))

    @_abc.abstractmethod
    def rem(self, divisor: _te.Self) -> _te.Self:
        ...

    def rem_euclid(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(type(self))
        return type(self)(_floor_division_remainder(self._value,
                                                    divisor._value))

    @_abc.abstractmethod
    def to_be_bytes(self) -> bytes:
        ...

    @_abc.abstractmethod
    def to_le_bytes(self) -> bytes:
        ...

    @_abc.abstractmethod
    def to_ne_bytes(self) -> bytes:
        ...

    __module__ = 'rustpy.primitive'

    def __new__(cls, _value: int) -> _te.Self:
        try:
            if not (cls.MIN._value <= _value <= cls.MAX._value):
                raise OverflowError(_value)
        except AttributeError:
            pass
        self = super().__new__(cls)
        self._value = _value
        return self

    @_t.overload
    def __and__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __and__(self, other: _t.Any) -> _t.Any:
        ...

    def __and__(self, other: _t.Any) -> _t.Any:
        return (type(self)(self._value & other._value)
                if isinstance(other, type(self))
                else NotImplemented)

    @_abc.abstractmethod
    def __invert__(self) -> _te.Self:
        ...

    def __int__(self) -> int:
        return self._value

    @_t.overload
    def __lshift__(self, other: u32) -> _te.Self:
        ...

    @_t.overload
    def __lshift__(self, other: _t.Any) -> _t.Any:
        ...

    def __lshift__(self, other: _t.Any) -> _t.Any:
        return (
            type(self).from_le_bytes(
                    (
                        ((self._value << u32_to_int(other % self.BITS))
                         & ((1 << u32_to_int(self.BITS)) - 1))
                    ).to_bytes(u32_to_int(self.BITS) // 8, 'little')
            )
            if isinstance(other, type(self.BITS))
            else NotImplemented
        )

    @_t.overload
    def __mod__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __mod__(self, other: _t.Any) -> _t.Any:
        ...

    def __mod__(self, other: _t.Any) -> _t.Any:
        return (type(self)(_trunc_division_remainder(self._value,
                                                     other._value))
                if isinstance(other, type(self))
                else NotImplemented)

    def __repr__(self) -> str:
        return f'{type(self).__qualname__}({self._value})'

    @_t.overload
    def __rshift__(self, other: u32) -> _te.Self:
        ...

    @_t.overload
    def __rshift__(self, other: _t.Any) -> _t.Any:
        ...

    def __rshift__(self, other: _t.Any) -> _t.Any:
        return (type(self)(self._value >> u32_to_int(other % self.BITS))
                if isinstance(other, type(self.BITS))
                else NotImplemented)

    def __str__(self) -> str:
        return f'{self._value}{type(self).__qualname__}'

    @_t.overload
    def __truediv__(self, other: _te.Self) -> _te.Self:
        ...

    @_t.overload
    def __truediv__(self, other: _t.Any) -> _t.Any:
        ...

    def __truediv__(self, other: _t.Any) -> _t.Any:
        return (type(self)(_trunc_division_quotient(self._value, other._value))
                if isinstance(other, type(self))
                else NotImplemented)


class BaseSignedInteger(_BaseInteger):
    @classmethod
    def from_be_bytes(cls, _bytes: bytes) -> _te.Self:
        if len(_bytes) != u32_to_int(cls.BITS) // 8:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        return cls(int.from_bytes(_bytes, 'big',
                                  signed=True))

    @classmethod
    def from_le_bytes(cls, _bytes: bytes) -> _te.Self:
        if len(_bytes) != u32_to_int(cls.BITS) // 8:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        return cls(int.from_bytes(_bytes, 'little',
                                  signed=True))

    @classmethod
    def from_ne_bytes(cls, _bytes: bytes) -> _te.Self:
        if len(_bytes) != u32_to_int(cls.BITS) // 8:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        return cls(int.from_bytes(_bytes, _sys.byteorder,
                                  signed=True))

    def abs(self) -> _te.Self:
        return type(self)(abs(self._value))

    def checked_abs(self) -> _Option[_te.Self]:
        try:
            return _Some(type(self)(abs(self._value)))
        except OverflowError:
            return _None()

    def checked_neg(self) -> _Option[_te.Self]:
        try:
            return _Some(type(self)(-self._value))
        except OverflowError:
            return _None()

    def is_negative(self) -> _bool:
        return _bool(self._value < 0)

    def is_positive(self) -> _bool:
        return _bool(self._value > 0)

    def neg(self) -> _te.Self:
        return type(self)(-self._value)

    def rem(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(divisor)
        return type(self)(_trunc_division_remainder(self._value,
                                                    divisor._value))

    def to_be_bytes(self) -> bytes:
        return self._value.to_bytes(u32_to_int(self.BITS) // 8, 'big',
                                    signed=True)

    def to_le_bytes(self) -> bytes:
        return self._value.to_bytes(u32_to_int(self.BITS) // 8, 'little',
                                    signed=True)

    def to_ne_bytes(self) -> bytes:
        return self._value.to_bytes(u32_to_int(self.BITS) // 8, _sys.byteorder,
                                    signed=True)

    def __invert__(self) -> _te.Self:
        return type(self)(~self._value)

    def __neg__(self) -> _te.Self:
        return type(self)(-self._value)


class BaseUnsignedInteger(_BaseInteger):
    @classmethod
    def from_be_bytes(cls, _bytes: bytes) -> _te.Self:
        if len(_bytes) != u32_to_int(cls.BITS) // 8:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        return cls(int.from_bytes(_bytes, 'big'))

    @classmethod
    def from_le_bytes(cls, _bytes: bytes) -> _te.Self:
        if len(_bytes) != u32_to_int(cls.BITS) // 8:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        return cls(int.from_bytes(_bytes, 'little'))

    @classmethod
    def from_ne_bytes(cls, _bytes: bytes) -> _te.Self:
        if len(_bytes) != u32_to_int(cls.BITS) // 8:
            raise TypeError(f'Invalid number of bytes, got {len(_bytes)}.')
        return cls(int.from_bytes(_bytes, _sys.byteorder))

    def rem(self, divisor: _te.Self) -> _te.Self:
        if not isinstance(divisor, type(self)):
            raise TypeError(divisor)
        return type(self)(_floor_division_remainder(self._value,
                                                    divisor._value))

    def to_be_bytes(self) -> bytes:
        return self._value.to_bytes(u32_to_int(self.BITS) // 8, 'big')

    def to_le_bytes(self) -> bytes:
        return self._value.to_bytes(u32_to_int(self.BITS) // 8, 'little')

    def to_ne_bytes(self) -> bytes:
        return self._value.to_bytes(u32_to_int(self.BITS) // 8, _sys.byteorder)

    def __invert__(self) -> _te.Self:
        return type(self)(self.MAX - self._value)


_SignedInteger = _t.TypeVar('_SignedInteger',
                            bound=BaseSignedInteger)


def signed_cls_to_max_value(cls: _t.Type[_SignedInteger]) -> _SignedInteger:
    return cls((1 << (u32_to_int(cls.BITS) - 1)) - 1)


def signed_cls_to_min_value(cls: _t.Type[_SignedInteger]) -> _SignedInteger:
    return cls(-(1 << (u32_to_int(cls.BITS) - 1)))


def u32_to_int(value: u32) -> int:
    return int(value)


_UnsignedInteger = _t.TypeVar('_UnsignedInteger',
                              bound=BaseUnsignedInteger)


def unsigned_cls_to_max_value(
        cls: _t.Type[_UnsignedInteger]
) -> _UnsignedInteger:
    return cls((1 << u32_to_int(cls.BITS)) - 1)


def unsigned_cls_to_min_value(
        cls: _t.Type[_UnsignedInteger]
) -> _UnsignedInteger:
    return cls(0)
